@@ -1,3 +1,3 @@
-from .coworks import Once, Every, At
 from .coworks import TechMicroService, BizFactory, BizMicroService, Blueprint
+from .coworks import hide_entry, Once, Every, At
 from .mixins import FileParam
