@@ -1,19 +1,20 @@
 import inspect
 import json
 import logging
-import sys
+import os
 import traceback
 from functools import update_wrapper
 from threading import Lock
-from typing import Dict
+from typing import Dict, List, Union
 
+import sys
 from aws_xray_sdk.core import xray_recorder
 from chalice import AuthResponse, BadRequestError, Rate, Cron
 from chalice import Chalice, Blueprint as ChaliceBlueprint
-
 from requests_toolbelt.multipart import MultipartEncoder
 
-from .config import Config
+from .config import Config, DEFAULT_WORKSPACE
+from .cws.error import CwsCommandError
 from .mixins import CoworksMixin, AwsSFNSession
 from .utils import begin_xray_subsegment, end_xray_subsegment
 from .utils import class_auth_methods, class_rest_methods, class_attribute, trim_underscores
@@ -22,32 +23,67 @@ logger = logging.getLogger()
 logger.setLevel(logging.INFO)
 
 
-class TechMicroService(CoworksMixin, Chalice):
-    """Simple tech microservice created directly from class."""
+class Blueprint(CoworksMixin, ChaliceBlueprint):
+    """ Represents a blueprint, list of routes that will be added to microservice when registered.
 
-    def __init__(self, app_name: str = None, config: Config = None, **kwargs):
+    See :ref:`Blueprint <blueprint>` for more information.
+
+    """
+
+    def __init__(self, **kwargs):
+        """Initialize a blueprint.
+
+        :param kwargs: Other Chalice parameters.
+
+        """
+        import_name = kwargs.pop('import_name', self.__class__.__name__)
+        super().__init__(import_name)
+
+    @property
+    def import_name(self):
+        return self._import_name
+
+    @property
+    def component_name(self):
+        return ''
+
+    @property
+    def current_app(self):
+        return self._current_app
+
+
+class TechMicroService(CoworksMixin, Chalice):
+    """Simple tech microservice created directly from class.
+    
+    See :ref:`tech` for more information.
+    
+    """
+
+    def __init__(self, ms_name: str = None, configs: Union[Config, List[Config]] = None, **kwargs):
         """ Initialize a technical microservice.
-        :param app_name: Name used to identify the resource.
-        :param config: Deployment configuration.
+        :param ms_name: Name used to identify the resource.
+        :param configs: Deployment configuration.
+        :param workspace used for execution.
         :param kwargs: Other Chalice parameters.
         """
-        app_name = app_name or self.__class__.__name__
-        authorizer = kwargs.pop('authorizer', None)
+        ms_name = ms_name or self.__class__.__name__
 
-        super().__init__(app_name, **kwargs)
+        self.configs = configs or [Config()]
+        if type(self.configs) is not list:
+            self.configs = [configs]
+        self.config = None
+
+        super().__init__(app_name=ms_name, **kwargs)
         self.experimental_feature_flags.update([
             'BLUEPRINTS'
         ])
 
-        self.config = config or Config()
+        # App and blueprints init deferered functions.
+        self.deferred_inits = []
+        self.blueprint_deferred_inits = []
 
-        self.__auth__ = self._create_auth_proxy(self, authorizer) if authorizer else None
-
-        # Blueprints added by names.
-        self.blueprints = {}
-
-        # Extensions defined by type's name (xriters, ..)
-        self.extensions = {}
+        # Extended commands added
+        self.commands = {}
 
         # A list of functions that will be called at the first activation.
         # To register a function, use the :meth:`before_first_request` decorator.
@@ -62,68 +98,103 @@ class TechMicroService(CoworksMixin, Chalice):
         self._got_first_activation = False
         self._before_activation_lock = Lock()
 
-        self.entries = {}
-        self._add_route()
+        self.entries = None
         self.sfn_call = False
 
         if "pytest" in sys.modules:
             xray_recorder.configure(context_missing="LOG_ERROR")
 
+        self.__global_auth__ = None
+
     @property
     def component_name(self):
         return class_attribute(self, 'url_prefix', '')
 
+    @property
+    def ms_name(self):
+        return self.app_name
+
     @property
     def ms_type(self):
         return 'tech'
 
-    @property
-    def lambda_zip_file(self):
-        return None
+    def deferred_init(self, workspace):
+        if self.entries is None:
+            self._init_routes(workspace=workspace)
+            for deferred_init in self.deferred_inits:
+                deferred_init(workspace)
+            for deferred_init in self.blueprint_deferred_inits:
+                deferred_init(workspace)
 
-    @property
-    def layer_zip_file(self):
-        return None
+    def register_blueprint(self, blueprint: Blueprint, authorizer=None, **kwargs):
+        """ Register a :class:`Blueprint` on the microservice.
 
-    def register_blueprint(self, blueprint, **kwargs):
+        :param blueprint:
+        :param authorizer:
+        :param kwargs:
+        :return:
+        """
         if 'name_prefix' not in kwargs:
             kwargs['name_prefix'] = blueprint.component_name
         if 'url_prefix' not in kwargs:
             kwargs['url_prefix'] = f"/{blueprint.component_name}"
 
-        self._add_route(blueprint, url_prefix=kwargs['url_prefix'])
-        super().register_blueprint(blueprint, **kwargs)
-        self.blueprints[blueprint.import_name] = blueprint
+        blueprint.__auth__ = self._create_auth_proxy(self, authorizer) if authorizer else None
 
-    def run(self, host='127.0.0.1', port=8000, project_dir='.', stage=None, debug=True):
-        # chalice.cli and .cws packages not defined in deployment
-        from chalice.cli import DEFAULT_STAGE_NAME
-        from .cws.factory import CwsCLIFactory
+        def deferred(workspace):
+            self._init_routes(workspace=workspace, component=blueprint, url_prefix=kwargs['url_prefix'])
+            Chalice.register_blueprint(self, blueprint, **kwargs)
 
-        if debug:
-            logging.basicConfig(stream=sys.stdout, level=logging.INFO, format='%(message)s')
+        self.blueprint_deferred_inits.append(deferred)
 
-        stage = stage or DEFAULT_STAGE_NAME
-        factory = CwsCLIFactory(self, project_dir, debug=debug)
-        config = factory.mock_config_obj(self, stage)
-        factory.run_local_server(self, config, host, port)
+    def execute(self, command, *, project_dir, module, workspace, output=None, error=None, **kwargs):
+        """Executes a client command."""
+        from coworks.cws.client import ProjectConfig
+        from coworks.cws.command import CwsCommandOptions
+
+        project_config = ProjectConfig(command, project_dir)
+        service = kwargs.setdefault('service', self.ms_name)
+        cmd = project_config.get_command(self, module, service, workspace)
+        if not cmd:
+            raise CwsCommandError(f"The command {command} was not added to the microservice {self.ms_name}.\n")
+
+        options = CwsCommandOptions(cmd, project_dir=project_dir, module=module, workspace=workspace, **kwargs)
+        project_config.complete_options(options)
+        cmd.execute(options=options, output=output, error=error)
+
+    def _init_routes(self, *, workspace=DEFAULT_WORKSPACE, component=None, url_prefix=None):
+        if self.config is None:
+            for conf in self.configs:
+                if conf.workspace == workspace:
+                    self.config = conf
+                    break
+            if self.config is None:
+                self.config = self.configs[0]
 
-    def _add_route(self, component=None, url_prefix=None):
         if component is None:
             component = self
 
-            # Adds class authorizer for every entries (if not already added before)
+        # External authorizer has priority (forced)
+        if self.config.auth:
+            if self.__global_auth__ is None:
+                self.__global_auth__ = self._create_auth_proxy(self, self.config.auth)
+            auth = component.__auth__ = self.__global_auth__
+        else:
             auth = class_auth_methods(component)
             if auth and component.__auth__ is None:
                 auth = TechMicroService._create_auth_proxy(component, auth)
                 component.__auth__ = auth
-        else:
-            # Adds the current_app auth for blueprints
-            auth = self.__auth__
+            auth = component.__auth__
 
         # Adds entrypoints
+        if self.entries is None:
+            self.entries = {}
         methods = class_rest_methods(component)
         for method, func in methods:
+            if getattr(func, '__cws_hidden', False):
+                continue
+
+            # Get function's route
             if func.__name__ == method:
                 route = f"{component.component_name}"
             else:
@@ -131,6 +202,8 @@ class TechMicroService(CoworksMixin, Chalice):
                 name = trim_underscores(name)  # to allow several functions with same route but different args
                 name = name.replace('_', '/')
                 route = f"{component.component_name}/{name}" if component.component_name else f"{name}"
+
+            # Get parameters
             args = inspect.getfullargspec(func).args[1:]
             defaults = inspect.getfullargspec(func).defaults
             varkw = inspect.getfullargspec(func).varkw
@@ -148,7 +221,7 @@ class TechMicroService(CoworksMixin, Chalice):
 
             # complete all entries
             component.route(f"/{route}", methods=[method.upper()], authorizer=auth, cors=self.config.cors,
-                            content_types=['multipart/form-data', 'application/json'])(proxy)
+                            content_types=['multipart/form-data', 'application/json', 'text/plain'])(proxy)
             if url_prefix:
                 self.entries[f"{url_prefix}/{route}"] = (method.upper(), func)
             else:
@@ -185,7 +258,12 @@ class TechMicroService(CoworksMixin, Chalice):
         return component.authorizer(name='auth')(proxy)
 
     def __call__(self, event, context):
+        # workspace = self._called_workspace(event, context)
+
         with self._before_activation_lock:
+            workspace = os.environ['WORKSPACE']
+            self.deferred_init(workspace=workspace)
+
             if not self._got_first_activation:
                 for func in self.before_first_activation_funcs:
                     func()
@@ -206,18 +284,18 @@ class TechMicroService(CoworksMixin, Chalice):
         # authorization call
         if event.get('type') == 'TOKEN':
             if self.debug:
-                print(f"Calling {self.app_name} for authorization")
+                print(f"Calling {self.ms_name} for authorization")
 
             if self.__auth__:
                 return self.__auth__(event, context)
 
-            print(f"Undefined authorization method for {self.app_name} ")
-            raise Exception('Unauthorized')
+            print(f"Undefined authorization method for {self.ms_name} ")
+            return 'Unauthorized', 403
 
         # step function call
         if event.get('type') == 'CWS_SFN':
             if self.debug:
-                print(f"Calling {self.app_name} by step function")
+                print(f"Calling {self.ms_name} by step function")
 
             self.sfn_call = True
             content_type = event['headers']['Content-Type']
@@ -233,7 +311,7 @@ class TechMicroService(CoworksMixin, Chalice):
                 raise BadRequestError(f"Undefined content type {content_type} for Step Function call")
         else:
             if self.debug:
-                print(f"Calling {self.app_name} with event {event}")
+                print(f"Calling {self.ms_name} with event {event}")
 
         res = super().__call__(event, context)
 
@@ -246,10 +324,21 @@ class TechMicroService(CoworksMixin, Chalice):
                 pass
 
         if self.debug:
-            print(f"Call {self.app_name} returns {res}")
+            print(f"Call {self.ms_name} returns {res}")
 
         return res
 
+    def deferred(self, f):
+        """Registers a function to be run once the microservice will be initialized.
+
+        May be used as a decorator.
+
+        The function will be called with one parameter, the workspace, and its return value is ignored.
+        """
+
+        self.deferred_inits.append(f)
+        return f
+
     def before_first_activation(self, f):
         """Registers a function to be run before the first activation of the microservice.
 
@@ -287,10 +376,11 @@ class TechMicroService(CoworksMixin, Chalice):
 
 
 class BizFactory(TechMicroService):
-    """Tech microservice to create, update and trigger biz microservices."""
+    """Tech microservice to create, update and trigger biz microservices.
+    """
 
     def __init__(self, sfn_name, **kwargs):
-        super().__init__(app_name=sfn_name, **kwargs)
+        super().__init__(ms_name=sfn_name, **kwargs)
 
         self.aws_profile = self.__sfn_client__ = self.__sfn_arn__ = None
         self.sfn_name = sfn_name
@@ -356,6 +446,7 @@ class BizFactory(TechMicroService):
 
         data = data or {}
         try:
+            self.biz[biz_name].data.update(data)
             return self.biz[biz_name](data, {})
         except KeyError:
             if self.debug:
@@ -368,7 +459,7 @@ class BizFactory(TechMicroService):
         if biz_name in self.biz:
             raise BadRequestError(f"Biz microservice {biz_name} already defined for {self.sfn_name}")
 
-        self.biz[biz_name] = BizMicroService(self, data, trigger, app_name=biz_name, **kwargs)
+        self.biz[biz_name] = BizMicroService(self, data, trigger, ms_name=biz_name, **kwargs)
         return self.biz[biz_name]
 
     def invoke(self, data):
@@ -394,7 +485,8 @@ class BizFactory(TechMicroService):
 
 
 class BizMicroService(TechMicroService):
-    """Biz composed microservice activated by a reactor."""
+    """Biz composed microservice activated by a reactor.
+    """
 
     def __init__(self, biz_factory, data, trigger, **kwargs):
         super().__init__(**kwargs)
@@ -406,24 +498,16 @@ class BizMicroService(TechMicroService):
         return self.biz_factory.invoke(self.data)
 
 
-class Blueprint(CoworksMixin, ChaliceBlueprint):
-    """Chalice blueprint created directly from class."""
+def hide_entry(f):
+    """Hide a route of the microservice.
 
-    def __init__(self, **kwargs):
-        import_name = kwargs.pop('import_name', self.__class__.__name__)
-        super().__init__(import_name)
+     May be used as a decorator.
 
-    @property
-    def import_name(self):
-        return self._import_name
+     Usefull when creating inherited microservice.
+     """
 
-    @property
-    def component_name(self):
-        return ''
-
-    @property
-    def current_app(self):
-        return self._current_app
+    setattr(f, '__cws_hidden', True)
+    return f
 
 
 class Once:
