@@ -11,6 +11,7 @@ from botocore.exceptions import BotoCoreError
 from chalice import BadRequestError, Response
 from requests_toolbelt.multipart import MultipartDecoder
 
+from coworks.cws.error import CwsError
 from coworks.utils import begin_xray_subsegment, end_xray_subsegment
 
 
@@ -19,6 +20,7 @@ class CoworksMixin:
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.debug = kwargs.pop('debug', False)
+        self.__auth__ = None
 
         self.aws_s3_sfn_data_session = AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
                                                     env_var_secret_key="AWS_RUN_SECRET_KEY",
@@ -37,9 +39,9 @@ class CoworksMixin:
             subsegment = begin_xray_subsegment(f"{func.__name__} microservice")
             try:
                 if subsegment:
-                    subsegment.put_metadata('headers', self.current_request.headers, "CoWorks")
+                    subsegment.put_metadata('headers', self.current_request.headers, "Coworks")
 
-                # Renames positionnal parameters (index added in label)
+                # Renames positional parameters (index added in label)
                 kwargs = {}
                 for kw, value in kws.items():
                     param = args[int(kw[1:])]
@@ -73,7 +75,7 @@ class CoworksMixin:
                                     name, content = self._get_multipart_content(part)
                                     add_param(name, content)
                             except Exception as e:
-                                print(e)
+                                raise CwsError(str(e))
                             kwargs = dict(**kwargs, **params)
 
                         elif content_type.startswith('application/json'):
@@ -141,8 +143,7 @@ class CoworksMixin:
             try:
                 s3_object = self.aws_s3_form_data_session.client.get_object(Bucket=pathes[0], Key=pathes[1])
             except BotoCoreError:
-                print(f"Bucket={pathes[0]} Key={pathes[1]} not found on s3")
-                return BadRequestError(f"Bucket={pathes[0]} Key={pathes[1]} not found on s3")
+                return CwsError(f"Bucket={pathes[0]} Key={pathes[1]} not found on s3")
             file = io.BytesIO(s3_object['Body'].read())
             mime_type = s3_object['ContentType']
         else:
@@ -231,11 +232,15 @@ class CoworksMixin:
 
 def _convert_response(resp):
     if type(resp) is tuple:
-        status_code = resp[1]
         if len(resp) == 2:
-            return Response(body=resp[0], status_code=status_code)
+            if type(resp[1]) is int:
+                return Response(body=resp[0], status_code=resp[1])
+            elif type(resp[1]) is dict:
+                return Response(body=resp[0], status_code=200, headers=resp[1])
+            else:
+                raise BadRequestError("Internal error (wrong result type)")
         else:
-            return Response(body=resp[0], status_code=status_code, headers=resp[2])
+            return Response(body=resp[0], status_code=resp[1], headers=resp[2])
 
     elif not isinstance(resp, Response):
         return Response(body=resp)
@@ -261,7 +266,6 @@ class Boto3Mixin:
     def aws_access_key(self):
         value = os.getenv(self.__env_var_access_key)
         if not value:
-            print(f"{self.__env_var_access_key} not defined in environment")
             raise EnvironmentError(f"{self.__env_var_access_key} not defined in environment")
         return value
 
@@ -269,7 +273,6 @@ class Boto3Mixin:
     def aws_secret_access_key(self):
         value = os.getenv(self.__env_var_secret_key)
         if not value:
-            print(f"{self.__env_var_secret_key} not defined in environment")
             raise EnvironmentError(f"{self.__env_var_secret_key} not defined in environment")
         return value
 
@@ -277,7 +280,6 @@ class Boto3Mixin:
     def region_name(self):
         value = os.getenv(self.__env_var_region)
         if not value:
-            print(f"{self.__env_var_region} not defined in environment")
             raise EnvironmentError(f"{self.__env_var_region} not defined in environment")
         return value
 
@@ -295,16 +297,14 @@ class Boto3Mixin:
                 try:
                     self.__session__ = boto3.Session(profile_name=self.__profile_name, region_name=region_name)
                 except Exception:
-                    print(f"Cannot create session for profile {self.__profile_name} and region {region_name}")
-                    raise
+                    raise CwsError(f"Cannot create session for profile {self.__profile_name} and region {region_name}")
             else:
                 access_key = self.aws_access_key
                 secret_key = self.aws_secret_access_key
                 try:
                     self.__session__ = boto3.Session(access_key, secret_key, region_name=region_name)
                 except Exception:
-                    print(f"Cannot create session for key {access_key}, secret {secret_key} and region {region_name}")
-                    raise
+                    raise CwsError(f"Cannot create session for key {access_key}, secret {secret_key} and region {region_name}")
         return self.__session__
 
 
