@@ -1,5 +1,6 @@
 from coworks import BizMicroService
 
+
 class BizMS(BizMicroService):
     def __init__(self):
-        super().__init__('step_function', app_name='biz')
+        super().__init__('step_function', ms_name='biz')
